// Commented out due to wrapper SDK also existing
type personId = string;
type roomId = string;
type messageId = string;
type orgId = string;
type roleId = string;
type memberId = string;
type teamId = string;
type teamMemberId = string;
type RoomTypes = "direct" | "group";

// Start types
interface SparkCallFeedback {
    userRating: number;
    userComments: string;
    includeLogs: boolean;
}
interface SparkCall {
    on(event:"ringing", callback:()=>void);
    on(event:"connected", callback:()=>void);
    on(event:"disconnected", callback:()=>void);
    on(event:"localMediaStream:change", callback:()=>void);
    on(event:"remoteMediaStream:change", callback:()=>void);
    on(event:"error", callback:(reason:string)=>void);
    
    readonly localMediaStream: MediaStream;
    readonly remoteMediaStream: MediaStream;
    /**
     * Disconnects this call session
     */     
    hangup() : Promise<undefined>;

    startSendingAudio() : Promise<undefined>;
    stopSendingAudio() : Promise<undefined>;
    
    startSendingVideo() : Promise<undefined>;
    stopSendingVideo() : Promise<undefined>;

    startReceivingAudio() : Promise<undefined>;
    stopReceivingAudio() : Promise<undefined>;
    
    startReceivingVideo() : Promise<undefined>;
    stopReceivingVideo() : Promise<undefined>;

    
    toggleSendingAudio() : Promise<undefined>;
    toggleReceivingAudio() : Promise<undefined>;

    toggleSendingVideo() : Promise<undefined>;
    toggleReceivingVideo() : Promise<undefined>;

    /**
     * Use to acknowledge (without answering) an incoming call. Will cause the
     * initiator's Call instance to emit the ringing event.
     */
    acknowledge() : Promise<undefined>;
    /**
     * Declines the inbound call
     * [Alias of reject]
     */
    decline() : Promise<undefined>;
    /**
     * rejects the inbound call
     */
    reject() : Promise<undefined>;
    /**
     * Accepts the call, with optional stream contraints
     */
    answer(options?:MediaStreamConstraints) : Promise<undefined>;
    /**
     * Sends feedback about this session
     */
    sendFeedback(feedback:SparkCallFeedback) : Promise<undefined>; 
}
interface SparkPerson {
    /**
     * ISO Date when user was created
     */
    created: string;
    /**
     * The ID of this person.
     */
    id: personId;
    /**
     * Emails linked to this account, usually only 1 exists
     */
    emails:  string[];
    /**
     * Full name of the user
     */
    displayName: string;
    /**
     * Short name of the user, usually first name
     */
    nickname: string;

    /**
     * [Optional]
     * URL to their image
     */
    avatar?: string;
    /**
     * [Optional]
     * The organisation this user may be part of.
     */
    orgId?: orgId;
    /**
     * [Optional]
     * ISO Date when user was last seen doing something in Spark
     */
    lastActivity?: string;
    /**
     * [Optional]
     * Never observed other keys than inactive, but is possible
     */ 
    status?: "inactive" | string;

    // Not in the SDK, only API for now (used for Admin API only)
    //roles?: roleId[];
    //licenses?: licenceId[];
}
interface SparkRoom {
    /**
     * The ID of this room
     */
    id: roomId;
    /**
     * The name of this room
     */
    title: string;
    /**
     * Checks if this room is locked
     */
    isLocked: boolean;
    /**
     * Type of the room. Rooms can be `direct` being 1 on 1 personal spaces,
     * or can be `group` which is your traditional space
     */
    type: RoomTypes;
    /**
     * If room is a part of a team, will have teamid here.
     */
    teamId?: teamId;
    /**
     * ISO Date of when this room was last used
     */
    lastActivity: string;
    /**
     * ISO Date of when this room was created
     */
    created: string;
    /**
     * The ID of the person that created this room
     */
    creatorId: personId;
    /**
     * The SIP Address of this room for phone purposes.
     */
    sipAddress: string;
}
interface SparkMessage {
    /**
     * [Server side]
     * The ID of this message
     */
    id?: messageId;
    /**
     * [Server side]
     * The Person sending this message
     */
    personId?: personId;
    /**
     * [Server side]
     * The email of the person sending this message
     */
    personEmail?: string;
    /**
     * The Room the message is in
     */
    roomId: roomId;
    /**
     * [Server side]
     * ISO Date the message was created.
     */
    created?: string;

    /**
     * [Optional]
     * The message in plain text
     */
    text?: string;
    /**
     * [Optional] [Experimental]
     * The message in raw html
     */
    html?: string;
    /**
     * [Optional] [Deprecated?]
     * The message in markdown syntax
     */
    markdown?: string;
    /**
     * [Optional]
     * the file URL's present in this message
     */
    files?: string[];
    /**
     * [Optional]
     * The people mentioned in this message
     */
    mentionedPeople?: personId[];
}
interface SparkMembership {
    /**
     * ISO Date of when this membership was made
     */
    created: string;
    /**
     * The ID of this membership
     */
    id: memberId;
    /**
     * Is this person a moderator of the room
     */
    isModerator: boolean;
    /**
     * TBD 
     */
    isMonitor: boolean;
    /**
     * The display name of the person
     */
    personDisplayName: string;
    /**
     * The primary email of the person
     */
    personEmail: string;
    /**
     * the ID of the person
     */
    personId: personId;
    /**
     * [Optional]
     * The organization the person is in
     */
    personOrgId?: orgId;
    /**
     * The room the person is in
     */
    roomId: roomId;
}
interface SparkTeam {
    /**
     * [Server side]
     * The ID of this team
     */
    id?: teamId;
    /**
     * The name of this team
     */
    name: string;
    /**
     * [Server side]
     * ISO Date of when this team was created
     */
    created?: string;
}
interface SparkTeamMembership {
    id: teamMemberId;
    personId: personId;
    personEmail: string;
    isModerator: boolean;
}
interface Page<T> {
    /**
     * The array containing the contents you are after
     */
    items: T[];
    /**
     * The length of `items`
     */
    length: number;
    // TODO: Might be something other than string, investigate
    links: string[];
    // Actually is `SparkProxy` but we don't need to care about that.
    spark: any;
}
// End types

// Start global properties
interface SparkPhone {
    /**
     * Checks the browser to see if WebRTC over h264 is supported
     */
    isCallingSupported() : Promise<boolean>;
    /**
     * Registers this phone to start listening for inbound calls
     */
    register() : Promise<undefined>;
    /**
     * Unregisters this phone so it won't receive calls anymore.
     */
    deregister() : Promise<undefined>;
    /**
     * Creates a local media stream based on options, defaults to both audio and video being used.
     */
    createLocalMediaStream(options?:{audio:boolean,video:boolean}|{constraints:{audio:boolean,video:boolean}}) : Promise<MediaStream>;
    /**
     * Dials a SIP uri or email, with some additional options available
     */
    dial(target:string, options?:{constraints?: {audio:boolean,video:boolean}, localMediaStream?:MediaStream}) : SparkCall;
    
}
interface SparkPeople {
    get(person: personId) : Promise<SparkPerson>;
    list(options: {email?: string, displayName?: string}) : Promise<Page<SparkPerson>>;
}
interface SparkRooms {
    /**
     * Creates a room with the title given, returns a promise of the room
     */
    create(options: {title: string}): Promise<SparkRoom>;
    /**
     * Returns a room based on id
     */
    get(id: roomId, options?: any) : Promise<SparkRoom>;
    /**
     * Lists all rooms the user has access to based on option criteria
     */
    list(options?:{teamId?: teamId, max?: number, type?: RoomTypes}) : Promise<Page<SparkRoom>>;
}
interface SparkMessages {
    /**
     * Creates a message to roomId or toPersonId with text or html or markdown
     */
    create(message: SparkMessage) : Promise<SparkMessage>;
    /**
     * Gets message details from ID (Mostly useless)
     */
    get(messageId: messageId) : Promise<SparkMessage>;
    /**
     * Gets messages from a room ID, with an optional max cap
     */
    // TODO: Figure out if the API options apply here too
    list(options?:{roomId: roomId, max?: number }) : Promise<Page<SparkMessage>>;
    /**
     * Deletes a message from the room
     */
    remove(message: messageId | SparkMessage) : Promise<undefined>;
}
interface SparkMemberships {
    /**
     * Creates a membership (used for adding people to rooms).
     */
    create(member:{personEmail: string, roomId:roomId}) : Promise<SparkMembership>;
    /**
     * Get member details based on id. (Mostly useless)
     */
    get(id: memberId) : Promise<SparkMembership>;
    /**
     * Gets a list of members based on personId or personEmail (get all the places you live in),
     * or roomId (get all the members in this room), and can filter based on max count.
     */
    list(options?: {personId?: personId, personEmail?: string, roomId?: roomId, max?: number}) : Promise<Page<SparkMembership>>;
    /**
     * Deletes a membership based on member id
     */
    remove(membership: SparkMembership | memberId) : Promise<undefined>;
    /**
     * Updates a membership, used mainly for moderator / monitor flags.
     */
    update(membership: SparkMembership) : Promise<SparkMembership>;
}
interface SparkTeams {
    /**
     * Creates a team, needs name to be present
     */
    create(team:SparkTeam) : Promise<SparkTeam>;
    /**
     * Gets a team by ID
     */
    get(id:teamId) : Promise<SparkTeam>;
    /**
     * Gets all teams you are a member of, has an optional max cap
     */
    list(options?: {max?: number}) : Promise<Page<SparkTeam>>;
    /**
     * Updates a team, only useful for name
     */
    update(team:SparkTeam) : Promise<SparkTeam>;
}
interface SparkTeamMemberships {
    create(membership) : Promise<SparkTeamMembership>;
    get(membership) : Promise<SparkTeamMembership>;
    list(options?: {max?: number}) : Promise<Page<SparkTeamMembership>>;
    remove(membership) : Promise<undefined>;
    update(membership) : Promise<SparkTeamMembership>;
}
interface CiscoSpark {
    phone: SparkPhone;
    people: SparkPeople;
    rooms: SparkRooms;
    messages: SparkMessages;
    memberships: SparkMemberships;
    teams: SparkTeams;
    teamMemberships: SparkTeamMemberships;
    webhooks: any;
}
// End global properties

// Start initialization
interface SparkAttrs {
    credentials : {
        access_token : string;
    };
}

interface SparkEntry {
    init(attrs?:SparkAttrs) : CiscoSpark;
}

declare var ciscospark: SparkEntry;
declare module "ciscospark" {
    export = ciscospark;
}
// End initialization